/***
*		DurRand.c -- articulation randomizer tool
*		 1997, 1998 Michael Rees (aka Another Option)   All rights reserved.
***/

/*
=============================================================================
|   DATE   | WHAT
=============================================================================
 29 Jan 97 | created
 19 Aug 97 | added support for live events in processevent() BUT the way it
           | is done doesn't acknowledge tool->ignore
 28 Feb 98 | real-time controllers
  1 Mar 98 | hacked in, BUT DIDN'T TEST, ccs and sysex in processlist()
*/


#define DBUG 0

//#define FILEDUMP

#include <exec/types.h>
#include <exec/memory.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <string.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <clib/alib_protos.h>
#include <clib/alib_stdio_protos.h>
#include <stdarg.h>

#include "bars.h"
#include "BarsMacros.h"
#include "BPLib.h"
#include "sysexmenu.h"
#include "configlib.h"
//#include "PopUpPipes.h"
#include "proto/durrand_protos.h"
#include "ToolID.h"
#include "CCNames.h"
#include "debug.h"

static UBYTE const versiontag[] = "$VER: DurRand 40.3 (01.03.98)";
#include "DurRandw.h"
#include "/icons/durrandicon.h"
#include "snapshotgadget.h"
#include "configgadget2.h"

#define CTRL_COUNT 4 //number of controllers

struct DurTool {
	struct Tool tool;
	ULONG flags;
	UWORD change;
	UWORD ignore;
	WORD maxdur;
	WORD mindur;
//	long on[128];
	struct ControllerInfo cinfo[CTRL_COUNT];
};

#define USE_SYSEX    1
#define FREE_SYSEX   2
#define USE_CONTROLS 4
#define SHOW_CCNUMS  8

#define MAX_DUR 12288   // largest resulting duration [16 bars oughta be enough?]
#define MAX_BEATS 1536  // range for pots, and divider for ccs
#define TOOL_SIZE sizeof(struct DurTool)
#define SX_LENGTH 12
#define CINFO_LENGTH (sizeof(struct ControllerInfo) * CTRL_COUNT)
#define tw tool->tool.window

extern struct IntuitionBase *IntuitionBase;
extern struct GfxBase *GfxBase;
extern struct LayersBase *LayersBase;
extern struct DOSBase *DOSBase;
extern struct ExecBase *SysBase;
extern struct Functions *functions;

/*==========================================================================
* inittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: sets tool defaults
*=========================================================================*/

static void __regargs inittool(struct DurTool *tool)

{
	tool->flags = FREE_SYSEX;
	tool->change = 384;
	tool->ignore = 96;
	tool->mindur = 48;
	tool->maxdur = 768;
	InitControlInfoArray(&tool->cinfo[0],CTRL_COUNT,CITYPE_NODIFF|CITYPE_OFF,
	                     CIFLAG_DISCARD);
	tool->tool.touched |= TOUCH_INIT;
}

/**********************************************/
/* twiddle and redraw config discard gadgets  */

static void __regargs refreshConfigGadgets(struct DurTool *tool)
{
	long i;
	struct Gadget *gadget;
	for (i=C_MAX_CHANGE ; i<=C_MIN ; i++) {
		gadget = functions->GetGadget(tw,i+CTRL_COUNT); //respective discard gadget
		if (tool->cinfo[i-C_MAX_CHANGE].flags & CIFLAG_DISCARD) {
			gadget->Flags |= SELECTED;
		}
		else {
			gadget->Flags &= ~SELECTED;
		}
		functions->RefreshGadget(tool->tool.window,i+CTRL_COUNT);
	}
}

/*************************************/
/* refresh real-time controller pots */

static void __regargs updateConfigPots(struct DurTool *tool)
{
	UBYTE i;
	
	for (i=C_MAX_CHANGE ; i<=C_MIN ; i++) {
		functions->ModifyEmbossedProp(tw,i,tool->cinfo[i-C_MAX_CHANGE].controller+1,
			                           0,0,0,0,0);
		functions->DrawEmbossedProp(tw,i);
		functions->DrawEmbossed(tw,i+GUP);
		functions->DrawEmbossed(tw,i+GDOWN);
	}
}

#define SECRET_MESSAGE 9999

/****************************************************************************/
/* note that MEMF_CLEAR in the myalloc() call screws up the Message somehow */
/****************************************************************************/
static void __regargs SendSecretMsg(struct Tool *tool,UWORD secret_message)
{
	struct Message *msg;

	if (msg = (struct Message *)functions->myalloc(sizeof(struct Message),0)) {
		msg->mn_Node.ln_Type = NT_MESSAGE;
		msg->mn_ReplyPort = NULL;
		msg->mn_Length = secret_message;
		PutMsg(tool->window->UserPort,msg);
	}
}

#define BITEME 0xFADE
#define SECRET_CC (SECRET_MESSAGE - 1)

/*********************************************************/

static struct Event * __regargs do_cc(struct DurTool *tool, struct Event *event)

{
	BYTE i = -1;      /* cinfo[] index */
	UBYTE gotone = 0; /* found entry for this cc# */
	UWORD x;

	if (event->byte1 > 120) return(event); /* none of the midi resets need apply */
//dprintf("docc() cc=%ld b2=%ld\n",event->byte1,event->byte2);
	/* else process controls */
	x = (event->byte2 * MAX_BEATS) / 127;
	while ( (i = FindController(&tool->cinfo[0],event->byte1,++i)) > -1 ) {
		gotone=1;
		if (tool->cinfo[i].flags & CIFLAG_DISCARD) {
			if (!InToolPad((struct Tool *)tool)) event->data = BITEME;
		}
		switch (i) {
			case 0 :  // max change
				tool->change = x;
				break;
			case 1 :  // ignore
				tool->ignore = x;
				break;
			case 2 :  // max
				tool->maxdur = x;
				break;
			case 3 :  // min
				tool->mindur = x;
				break;
		}
	}
	if (event->data == BITEME) {
		functions->freeevent(event);
		event=NULL;
	}
	if (gotone && tw && !InToolPad((struct Tool *)tool)) 
		SendSecretMsg(&tool->tool,SECRET_CC);
	return(event);
}

/*==========================================================================
* dosysex
*=========================================================================*/

static __regargs struct Event *dosysex(struct DurTool *tool,
                                       struct StringEvent *event)
{
	register unsigned char *sxdata;
	UWORD sxflag;

	if (sxdata = checksysex(event,TOOL_SYSEX_ID,tool->tool.toolid)) {
		sxflag = tool->flags & (USE_SYSEX | FREE_SYSEX);
		memcpy((void *)&tool->flags,(void *)sxdata,SX_LENGTH);
		memcpy((void *)&tool->cinfo,(void *)&sxdata[SX_LENGTH],CINFO_LENGTH);
		if (sxflag & USE_SYSEX) tool->flags |= USE_SYSEX;
		if (sxflag & FREE_SYSEX) tool->flags |= FREE_SYSEX;
		if (!InToolPad((struct Tool *)tool)) {
			if (tool->tool.window) SendSecretMsg(&tool->tool,SECRET_MESSAGE);
			if (tool->flags & FREE_SYSEX) {
				functions->freestring(event->string);
				functions->freeevent((struct Event *)event);
				event=NULL;
			}
		}
	}
	return((struct Event *)event);
}

/***************************************/
static UWORD __regargs doit(struct DurTool *tool, struct NoteEvent *event)
{
	register long change;

	change = (functions->random(99) < 50) ? -1 : 1;
	change *= functions->random(tool->change);
	change += event->duration;
	if (change < 1) change = 1;
	if (change < tool->mindur) change = tool->mindur;
	if (tool->maxdur <= MAX_BEATS) {
		if (change > tool->maxdur) change = tool->maxdur;
	}
	else if (change > MAX_DUR) {
		change = MAX_DUR;
	}
	return((UWORD)change);
}

/*==========================================================================
* processevent
*
*  INPUT: pointer to Event
* RETURN: pointer to Event
*
* EFFECT: processes events one at a time
*=========================================================================*/

static struct Event * __stdargs __saveds processevent(struct NoteEvent *event)

{
   struct DurTool *tool = (struct DurTool *)event->tool;

	if (!tool->tool.touched) inittool(tool);
	event->tool = tool->tool.next;
	if (IsNoteOn(event)) {
		if (event->duration) {
			if (event->duration > tool->ignore) {
//				doit(tool,event);
				event->duration = doit(tool,event);
				event->next = CloneNoteOff(event);
				if (!event->next) {
					functions->freeevent((struct Event *)event);
					event = NULL;
				}
			}
		}
		/* support for live events here if you so desire (I did),
		   BUT it disregards tool->ignore! */
		else {
			event->next = CloneNoteOff(event);
			if (!event->next) {
					functions->freeevent((struct Event *)event);
					event = NULL;
			}
			else event->next->time += doit(tool,event);
		}
	}
	else if (IsNoteOff(event)) {
		functions->freeevent((struct Event *)event);
		event = NULL;
	}
	else if (IsCtrlChg(event) && (tool->flags & USE_CONTROLS)) {
		event = (struct NoteEvent *)do_cc(tool,(struct Event *)event);
	}
	else if (IsSysEx(event) && (tool->flags & USE_SYSEX)) {
		event = (struct NoteEvent *)dosysex(tool,(struct StringEvent *)event);
	}
	return((struct Event *)event);
}

/*==========================================================================
* processlist
*=========================================================================*/

static struct Event * __stdargs __saveds processlist(struct DurTool *tool,
                                                     struct NoteEvent *list,
                                                     long start, long stop)
{
	struct NoteEvent *event;
	ULONG flags;
	UWORD change;
	UWORD ignore;
	WORD maxdur;
	WORD mindur;

	if (!tool->tool.touched) inittool(tool);
	if (start < 0) start = 0;
	/* push */
	if (tool->flags & USE_SYSEX) flags = tool->flags;
	if (tool->flags & USE_CONTROLS) {
		change = tool->change;
		ignore = tool->ignore;
		maxdur = tool->maxdur;
		mindur = tool->mindur;
	}
	event = list;
	while (event->time < start) event = event->next;
	while (event->time < stop) {
		event->tool = tool->tool.next;
		if (IsNoteOn(event)) {
			if (event->duration > tool->ignore) {
//				doit(tool,event);
				event->duration = doit(tool,event);
			}
		}
		else if (IsCtrlChg(event) && (tool->flags & USE_CONTROLS)) {
			do_cc(tool,(struct Event *)event);
		}
		else if (IsSysEx(event) && (tool->flags & USE_SYSEX)) {
			dosysex(tool,(struct StringEvent *)event);
		}
		event = event->next;
	}
	/* pop */
	if (tool->flags & USE_SYSEX) tool->flags = flags;
	if (tool->flags & USE_CONTROLS) {
		tool->change = change;
		tool->ignore = ignore;
		tool->maxdur = maxdur;
		tool->mindur = mindur;
	}
	return((struct Event *)list);
}

struct DurTool *selectedtool = NULL;
/***
 *** displays time m.bb.ccc, 0=='Nothing', '<' preceding time
 ***/

static long __stdargs __saveds hiloBeatMe(struct Window *w,struct Gadget *g,long x)
{
	char text[13];

	if (x==0) {
		strcpy(text,"   Nothing ");
	}
	else {
		functions->lengthtostring(selectedtool->tool.clip,0,x,text);
		text[2] = '<';
	}
	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset+5, g->TopEdge+YOffset,
            HILO_APEN,HILO_BPEN, HILO_DRAWMODE);
	Text(w->RPort,&text[2],9);
	return((long)x);
}

/***
 *** displays time m.bb.ccc, x==MAX_BEATS == No maximum
 ***/

static long __stdargs __saveds hiloBeatOff(struct Window *w,struct Gadget *g,long x)
{
	char text[13];

	if (x==MAX_BEATS+1) {
		strcpy(text,"The Big Sky");
	}
	else {
		functions->lengthtostring(selectedtool->tool.clip,0,x,text);
	}
	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset+13, g->TopEdge+YOffset,
            HILO_APEN,HILO_BPEN, HILO_DRAWMODE);
	Text(w->RPort,&text[3],8);
	return((long)x);
}

/***
 *** displays time m.bb.ccc
 ***/

static long __stdargs __saveds hiloBeat(struct Window *w,struct Gadget *g,long x)
{
	char text[13];

	functions->lengthtostring(selectedtool->tool.clip,0,x,text);
	SetRPort(w->RPort, g->LeftEdge+g->Width+XOffset+13, g->TopEdge+YOffset,
            HILO_APEN,HILO_BPEN, HILO_DRAWMODE);
	Text(w->RPort,&text[3],8);
	return((long)x);
}

/*==========================================================================
* dosnapshot
*=========================================================================*/

static void __regargs dosnapshot(struct DurTool *tool)
{
	UBYTE sx[SX_LENGTH];
	struct StringEvent *event;
	
	memcpy(sx,&tool->flags,SX_LENGTH);
	if (event = BuildSnapshot(&tool->tool, 
	                          functions->running ? functions->timenow : -1,
	                          sx, SX_LENGTH)) {
		PostSnapshot(&tool->tool,event);
	}
}

/* pens for config pot value text */
#define TEXTCOLOR 1
#define BACKCOLOR 0

/* display pot value where 0 is Off */
static long __stdargs __saveds hiloOff(struct Window *w, struct Gadget *g, long h)
{
	char text[8];
	
	Move(w->RPort,g->LeftEdge+g->Width+XOffset,g->TopEdge+YOffset);
	SetAPen(w->RPort,TEXTCOLOR);
	SetBPen(w->RPort,BACKCOLOR);
	SetDrMd(w->RPort,JAM2);
	if (--h > -1) {
		if (selectedtool->flags & SHOW_CCNUMS) {
#pragma msg 104 ignore push
			sprintf(text,"%3ld",h);
#pragma msg 104 ignore pop
			strcat(&text[3],"    ");
			Text(w->RPort,text,7);
		}
		else {
			Text(w->RPort,ccnames[h],7);
		}
	}
	else {
#pragma msg 104 ignore push
		sprintf(text,"%s","Off");
#pragma msg 104 ignore pop
		strcat(&text[3],"    ");
		Text(w->RPort,text,7);
	}
	return(h);
}

#define REFRESH_CONFIG     0x40
#define REFRESH_DRAWCONFIG 0x80

/*==========================================================================
* edittool
*
*  INPUT: pointer to tool
* RETURN: none
*
* EFFECT: ui handler
*=========================================================================*/

static void __stdargs __saveds edittool(struct DurTool *tool)

{
	struct IntuiMessage *message;
	long class; 
	long code;
	struct Gadget *gadget;
	struct NewWindow *nw;
	struct Menu *menu;
	struct MenuItem *menuitem;
	UBYTE refresh;

	if (!(tool->tool.touched & TOUCH_INIT)) inittool(tool);
	if (tool->flags & USE_SYSEX) MenuItem1.Flags |= CHECKED;
	else MenuItem1.Flags &= ~CHECKED;
	if (tool->flags & FREE_SYSEX) MenuItem2.Flags |= CHECKED;
	else MenuItem2.Flags &= ~CHECKED;
	if (tool->flags & USE_CONTROLS) MenuItem4.Flags |= CHECKED;
	else MenuItem4.Flags &= ~CHECKED;
	if (tool->flags & SHOW_CCNUMS) MenuItem5.Flags |= CHECKED;
	else MenuItem5.Flags &= ~CHECKED;
	if (InToolPad((struct Tool *)tool)) {
		refresh = 20;
		MenuItem2.Flags &= ~ITEMENABLED;
		Gadget13.Flags |= GADGDISABLED;
		Gadget14.Flags |= GADGDISABLED;
		Gadget15.Flags |= GADGDISABLED;
		Gadget16.Flags |= GADGDISABLED;
	}
	else {
		refresh = 40;
		MenuItem2.Flags |= ITEMENABLED;
		Gadget13.Flags &= ~GADGDISABLED;
		Gadget14.Flags &= ~GADGDISABLED;
		Gadget15.Flags &= ~GADGDISABLED;
		Gadget16.Flags &= ~GADGDISABLED;
	}
	nw=OpenEmbossedWindowSlop(&NewWindowStructure1,&tool->tool,0,WTITLE,refresh);
	if (!nw) return;
	AddConfigGadget(tw);
	if (refresh == 40) {
		AddSnapshotGadget(tw);
	}
	if (menu = functions->DupeMenu(&MenuList1)) SetMenuStrip(tw,menu);
	else SetWindowTitles(tw,(UBYTE *)-1,MenuList1.MenuName);
	functions->FatEmbossedPropOn(tw,MAX_CHANGE,MAX_CHANGE+GDOWN,MAX_CHANGE+GUP,
	                             hiloBeat,MAX_BEATS+1,1);
	functions->FatEmbossedPropOn(tw,MAX,MAX+GDOWN,MAX+GUP,hiloBeatOff,MAX_BEATS+2,1);
	functions->FatEmbossedPropOn(tw,MIN,MIN+GDOWN,MIN+GUP,hiloBeat,MAX_BEATS+1,1);
	functions->FatEmbossedPropOn(tw,IGNORE,IGNORE+GDOWN,IGNORE+GUP,hiloBeatMe,
	                             MAX_BEATS+1,1);
	for (code = C_MAX_CHANGE ; code <= C_MIN ; code++) { //emboss config pots
		functions->FatEmbossedPropOn(tw,code,code+GDOWN,code+GUP,hiloOff,122,1);
	}
	if (WindowIsTall(tw)) {
		DrawConfigFrame(tw);
		refresh = REFRESH_CONFIG | REFRESH_DRAWCONFIG;
	}
	else refresh = 0;
	refresh |= 1;
	/*** ui handling ***/
	for ( ; ; ) {
		if (refresh) {
			if (refresh & 1) {
				selectedtool = tool;
				functions->ModifyEmbossedProp(tw,MAX_CHANGE,tool->change,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,MAX_CHANGE);
				if (tool->maxdur < 1) tool->maxdur = 1;
				if (tool->maxdur < MAX_BEATS+1) {
					if (tool->maxdur <= tool->mindur) tool->maxdur = tool->mindur+1;
					if (tool->maxdur > MAX_BEATS) tool->maxdur = MAX_BEATS;
					if (tool->mindur >= tool->maxdur) tool->mindur = tool->maxdur-1;
				}
				if (tool->mindur < 1) tool->mindur = 1;
				functions->ModifyEmbossedProp(tw,MAX,tool->maxdur,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,MAX);
				functions->ModifyEmbossedProp(tw,MIN,tool->mindur,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,MIN);
				functions->ModifyEmbossedProp(tw,IGNORE,tool->ignore,0,0,0,0,0);
				functions->DrawEmbossedProp(tw,IGNORE);
			}
			if (refresh & REFRESH_CONFIG) {
				UpdateControllers(&tool->cinfo[0]);
				updateConfigPots(tool);
			}
			if (refresh & REFRESH_DRAWCONFIG) {
				refreshConfigGadgets(tool);
			}
			refresh = 0;
		}
		message = (struct IntuiMessage *)functions->GetIntuiMessage(tw);
		if (message->ExecMessage.mn_Length == SECRET_MESSAGE)
			refresh = 0xFF;
		else if (message->ExecMessage.mn_Length == SECRET_CC)
			refresh = 0xFF & ~(REFRESH_CONFIG | REFRESH_DRAWCONFIG);
		if (refresh) {
			functions->myfree((char *)message,sizeof(struct Message));
			continue;
		}
		/* else */
		class = message->Class;
		code = message->Code;
		gadget = (struct Gadget *)message->IAddress;
		ReplyMsg((struct Message *)message);
		class = functions->SystemGadgets(tw,class,gadget,code);

		if (class == CLOSEWINDOW) {
			break;
		}

		else if (class == NEWSIZE) {
			if (WindowIsTall(tw)) {
				DrawConfigFrame(tw);
				refresh = REFRESH_CONFIG | REFRESH_DRAWCONFIG;
			}
			if (InToolPad((struct Tool *)tool)) code = -20;
			else code = -40;
			SizeTitleBar(tw,code);
			functions->RefreshGadget(tw,CONFIG_GADGET_ID);
			if (code==-40) functions->RefreshGadget(tw,SNAPSHOT_GADGET_ID);
		}

		else if (class == MENUPICK) {
			while (code != MENUNULL) {
				menuitem = ItemAddress(menu,code);
				switch (ITEMNUM(code)) {
					case 0 :
						tool->flags ^= USE_SYSEX;
						break;
					case 1 :
						tool->flags ^= FREE_SYSEX;
						break;
					case 2 :
						dosnapshot(tool);
						break;
					case 3 :
						tool->flags ^= USE_CONTROLS;
						break;
					case 4 :
						tool->flags ^= SHOW_CCNUMS;
						updateConfigPots(tool);
						break;
				}
				code = menuitem->NextSelect;
			}
		}

		else if (class == GADGETUP) {
			switch (gadget->GadgetID) {
				case SNAPSHOT_GADGET_ID :
					dosnapshot(tool);
					break;
				case CONFIG_GADGET_ID :
					ZoomWindow(tw);
					break;
				case DIS_CHANGE :
				case DIS_IGNORE :
				case DIS_MAX :
				case DIS_MIN :
					tool->cinfo[gadget->GadgetID - DIS_CHANGE].flags ^= CIFLAG_DISCARD;
					refresh = REFRESH_CONFIG | REFRESH_DRAWCONFIG;
					break;
			}
		}

		else if (class == GADGETDOWN) {
			selectedtool = tool;
			switch (gadget->GadgetID) {
				case C_MAX_CHANGE :
				case C_IGNORE :
				case C_MAX :
				case C_MIN :
					tool->cinfo[gadget->GadgetID - C_MAX_CHANGE].controller =
						functions->DragEmbossedProp(tw,gadget->GadgetID);
					refresh = REFRESH_CONFIG;
					break;

				case C_MAX_CHANGE+GDOWN :
				case C_IGNORE+GDOWN :
				case C_MAX+GDOWN :
				case C_MIN+GDOWN :
					tool->cinfo[gadget->GadgetID - C_MAX_CHANGE - GDOWN].controller =
						   functions->ShiftEmbossedProp(tw,gadget->GadgetID-GDOWN,-1,0);
					refresh = REFRESH_CONFIG;
					break;

				case C_MAX_CHANGE+GUP :
				case C_IGNORE+GUP :
				case C_MAX+GUP :
				case C_MIN+GUP :
					tool->cinfo[gadget->GadgetID - C_MAX_CHANGE - GUP].controller =
						      functions->ShiftEmbossedProp(tw,gadget->GadgetID-GUP,1,0);
					refresh = REFRESH_CONFIG;
					break;

				case MAX_CHANGE :
					tool->change = functions->DragEmbossedProp(tw,MAX_CHANGE);
					refresh = 1;
					break;
				case MAX_CHANGE+GDOWN :
					tool->change = functions->ShiftEmbossedProp(tw,MAX_CHANGE,-1,0);
					refresh = 1;
					break;
				case MAX_CHANGE+GUP :
					tool->change = functions->ShiftEmbossedProp(tw,MAX_CHANGE,1,0);
					refresh = 1;
					break;

				case MAX :
					tool->maxdur = functions->DragEmbossedProp(tw,MAX);
					refresh = 1;
					break;
				case MAX+GDOWN :
					tool->maxdur = functions->ShiftEmbossedProp(tw,MAX,-1,0);
					refresh = 1;
					break;
				case MAX+GUP :
					tool->maxdur = functions->ShiftEmbossedProp(tw,MAX,1,0);
					refresh = 1;
					break;

				case MIN :
					tool->mindur = functions->DragEmbossedProp(tw,MIN);
					refresh = 1;
					break;
				case MIN+GDOWN :
					tool->mindur = functions->ShiftEmbossedProp(tw,MIN,-1,0);
					refresh = 1;
					break;
				case MIN+GUP :
					tool->mindur = functions->ShiftEmbossedProp(tw,MIN,1,0);
					refresh = 1;
					break;

				case IGNORE :
					tool->ignore = functions->DragEmbossedProp(tw,IGNORE);
					refresh = 1;
					break;
				case IGNORE+GDOWN :
					tool->ignore = functions->ShiftEmbossedProp(tw,IGNORE,-1,0);
					refresh = 1;
					break;
				case IGNORE+GUP :
					tool->ignore = functions->ShiftEmbossedProp(tw,IGNORE,1,0);
					refresh = 1;
					break;

				case POPUP_CHANGE :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->change = 768 >> code;
						refresh = 1;
					}
					break;
				case POPUP_MAX :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->maxdur = 768 >> code;
						refresh = 1;
					}
					break;
				case POPUP_MIN :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->mindur = 768 >> code;
						refresh = 1;
					}
					break;
				case POPUP_IGNORE :
					code = functions->popupnote(tw,-1);
					if (code >= 0) {
						tool->ignore = 768 >> code;
						refresh = 1;
					}
					break;
			}
		}
	}
	selectedtool = NULL;
	if (menu) functions->DeleteMenu(menu);
	functions->FatEmbossedPropOff(tw,MAX_CHANGE,MAX_CHANGE+GDOWN,MAX_CHANGE+GUP);
	functions->FatEmbossedPropOff(tw,MAX,MAX+GDOWN,MAX+GUP);
	functions->FatEmbossedPropOff(tw,MIN,MIN+GDOWN,MIN+GUP);
	functions->FatEmbossedPropOff(tw,IGNORE,IGNORE+GDOWN,IGNORE+GUP);
	for (code = C_MAX_CHANGE ; code <= C_MIN ; code++) { //emboss config pots
		functions->FatEmbossedPropOff(tw,code,code+GDOWN,code+GUP);
	}
	NewCloseEmbossedWindow(&tool->tool,nw);
}

static struct Tool * __stdargs __saveds loadtool(long file, long size)
{
	struct DurTool *tool;

	if (tool = (struct DurTool *)functions->myalloc(TOOL_SIZE,0)) {
		functions->fastread(file,(char *)tool,size);
		if (size != TOOL_SIZE) { // must be an old tool without ccs
			InitControlInfoArray(&tool->cinfo[0],CTRL_COUNT,CITYPE_NODIFF|CITYPE_OFF,
	                           CIFLAG_DISCARD);
			tool->flags &= ~USE_CONTROLS;
		}
		size -= TOOL_SIZE;
		if (size > 0) functions->fastseek(file,size,0);
	}
	return((struct Tool *)tool);
}			


struct ToolMaster master;

/*==========================================================================
* inittoolmaster
*
*  INPUT: none
* RETURN: pointer to ToolMaster
*
* EFFECT: initializes.
*=========================================================================*/

struct ToolMaster * __stdargs __saveds inittoolmaster(void)

{
	memset((char *)&master,0,sizeof(struct ToolMaster));
	master.toolsize = TOOL_SIZE;
	master.toolid = RDur_TOOLID;
	master.edittool = edittool;
	master.processevent = processevent;
	master.processlist = processlist;
	master.loadtool = loadtool;
	master.image = &IconDown;
	strcpy(master.name,WTITLE);
	master.tooltype = TOOL_NORMAL;
	return(&master);
}
